"""Asynchronous event tracking for Mixpanel"""

VERSION = (0, 6, 2, '')

__version__ = ".".join(map(str, VERSION[:-1]))
__release__ = ".".join(map(str, VERSION))
__author__ = "Wes Winham"
__contact__ = "winhamwr@gmail.com"
__homepage__ = "http://winhamwr.github.io/mixpanel-celery/"
__docformat__ = "restructuredtext"
